@@ -1,20 +1,19 @@
 use crate::{
     config::Config,
-    discref::{Device, Page, PageOps, ReadPage, UpdateList},
+    device::{Device, Page, PageOps, ReadPage, UpdateList},
     error::PERes,
     flush_checksum::{double_buffer_check, write_root_page},
-    io::{read_u64, write_u64, InfallibleReadFormat},
+    io::{read_u64, write_u64, InfallibleRead, InfallibleReadFormat},
 };
 use linked_hash_map::LinkedHashMap;
 use std::{
-    io::Read,
     sync::Mutex,
     time::{Duration, Instant},
 };
 
 const ALLOCATOR_PAGE_EXP: u8 = 10; // 2^10
-const ALLOCATER_ROOT_PAGE_VERSION_V0: u8 = 0;
-const ALLOCATER_ROOT_PAGE_VERSION: u8 = ALLOCATER_ROOT_PAGE_VERSION_V0;
+const ALLOCATOR_ROOT_PAGE_VERSION_V0: u8 = 0;
+const ALLOCATOR_ROOT_PAGE_VERSION: u8 = ALLOCATOR_ROOT_PAGE_VERSION_V0;
 
 struct CacheEntry {
     page: ReadPage,
@@ -88,39 +87,23 @@ impl Cache {
 
 struct FreeList {
     list: [u64; 32],
-    last_flush: u8,
     to_flush: bool,
 }
 impl FreeList {
-    fn new(list: [u64; 32], last_flush: u8) -> Self {
-        FreeList {
-            list,
-            last_flush,
-            to_flush: false,
-        }
+    fn new(list: [u64; 32]) -> Self {
+        FreeList { list, to_flush: false }
     }
 
-    fn read(page: &mut ReadPage) -> PERes<FreeList> {
+    fn read(page: &mut ReadPage, holder: &mut RootPageHolder, counter: &mut Counter) -> PERes<FreeList> {
         match page.read_u8() {
-            ALLOCATER_ROOT_PAGE_VERSION_V0 => Self::read_v0(page),
+            ALLOCATOR_ROOT_PAGE_VERSION_V0 => Self::read_v0(page, holder, counter),
             _ => panic!("unsupported format"),
         }
     }
-    fn read_v0(page: &mut ReadPage) -> PERes<FreeList> {
-        let mut buffer_0 = [0; 259];
-        let mut buffer_1 = [0; 259];
-        let freelist;
-        let last_flush;
-        page.read_exact(&mut buffer_0)?;
-        page.read_exact(&mut buffer_1)?;
-        let (flush_number, first) = double_buffer_check(&buffer_0, &buffer_1);
-        last_flush = flush_number;
-        if first {
-            freelist = Self::read_free_list(&buffer_0);
-        } else {
-            freelist = Self::read_free_list(&buffer_1);
-        }
-        Ok(FreeList::new(freelist, last_flush))
+    fn read_v0(page: &mut ReadPage, holder: &mut RootPageHolder, counter: &mut Counter) -> PERes<FreeList> {
+        let buffer = Allocator::read_root_page_int(page, 259, holder, counter);
+        let freelist = Self::read_free_list(&buffer);
+        Ok(FreeList::new(freelist))
     }
 
     pub fn read_free_list(buffer: &[u8]) -> [u64; 32] {
@@ -141,10 +124,8 @@ impl FreeList {
         buffer
     }
 
-    pub fn write_list(&mut self, page: &mut Page) -> PERes<()> {
-        let mut buffer = Self::write_free_list(&self.list);
-        self.last_flush = write_root_page(page, &mut buffer, ALLOCATER_ROOT_PAGE_VERSION, self.last_flush)?;
-        Ok(())
+    pub fn write_list(&mut self) -> [u8; 259] {
+        Self::write_free_list(&self.list)
     }
 
     pub fn set(&mut self, exp: u8, new_page: u64) -> u64 {
@@ -157,13 +138,53 @@ impl FreeList {
         self.list[exp as usize]
     }
 }
+
+struct RootWriteInfo {
+    page: u64,
+    buffer: Vec<u8>,
+    version: u8,
+}
+#[derive(Default)]
+struct RootPageHolder {
+    page: u64,
+    buffer: Option<Vec<u8>>,
+    dirty: bool,
+    version: u8,
+}
+impl RootPageHolder {
+    fn write_data(&mut self) -> Option<RootWriteInfo> {
+        if let Some(buff) = &self.buffer {
+            if self.dirty {
+                self.dirty = false;
+                return Some(RootWriteInfo {
+                    page: self.page,
+                    buffer: buff.clone(),
+                    version: self.version,
+                });
+            }
+        }
+        None
+    }
+}
+
+#[derive(Default)]
+struct Counter {
+    flush_counter: u8,
+}
+#[derive(Default)]
+struct FlushCount {
+    free_list: Counter,
+    journal: Counter,
+    address: Counter,
+}
+
 // Root pages monitor to avoid to write a root page before the relative backup root page is not
 // flushed
 #[derive(Default)]
 struct RootMonitor {
-    free_list_dirty: bool,
-    journal_dirty: bool,
-    address_dirty: bool,
+    free_list_holder: RootPageHolder,
+    journal_holder: RootPageHolder,
+    address_holder: RootPageHolder,
 }
 
 // TODO: Manage defragmentation by merging/splitting pages in the free list
@@ -172,13 +193,16 @@ pub struct Allocator {
     free_list: Mutex<FreeList>,
     cache: Mutex<Cache>,
     root_monitor: Mutex<RootMonitor>,
+    flush_count: Mutex<FlushCount>,
     page: u64,
 }
 
 impl Allocator {
     pub fn new(dr: Box<dyn Device>, config: &Config, page: u64) -> PERes<Allocator> {
+        let mut root_monitor = RootMonitor::default();
+        let mut flush_count = FlushCount::default();
         let mut pg = dr.load_page(page)?;
-        let mut freelist = FreeList::read(&mut pg)?;
+        let mut freelist = FreeList::read(&mut pg, &mut root_monitor.free_list_holder, &mut flush_count.free_list)?;
 
         // This may cause disk leaks in case of crash, but
         // keep the system safe
@@ -198,15 +222,18 @@ impl Allocator {
             disc: dr,
             free_list: Mutex::new(freelist),
             cache: Mutex::new(Cache::new(cache_size, cache_age_limit)),
-            root_monitor: Default::default(),
+            root_monitor: Mutex::new(root_monitor),
+            flush_count: Mutex::new(flush_count),
             page,
         })
     }
 
     pub fn init(dr: Box<dyn Device>, config: &Config) -> PERes<(u64, Allocator)> {
         let mut page = dr.create_page(ALLOCATOR_PAGE_EXP)?;
-        let mut list = FreeList::new([0; 32], 0);
-        list.write_list(&mut page)?;
+        let mut list = FreeList::new([0; 32]);
+        let mut counter = Counter::default();
+        let buffer = list.write_list();
+        Allocator::write_root_page(&mut page, &mut counter, buffer.to_vec(), ALLOCATOR_ROOT_PAGE_VERSION)?;
         dr.flush_page(&page)?;
         let allocate_page = page.get_index();
         Ok((allocate_page, Allocator::new(dr, config, allocate_page)?))
@@ -298,6 +325,11 @@ impl Allocator {
         Ok(page)
     }
 
+    pub fn flush_journal(&self, page: &Page) -> PERes<()> {
+        self.disc.flush_page(page)?;
+        Ok(())
+    }
+
     pub fn flush_page(&self, page: Page) -> PERes<()> {
         self.disc.flush_page(&page)?;
         {
@@ -374,42 +406,86 @@ impl Allocator {
         let mut lock = self.free_list.lock()?;
         if lock.to_flush {
             let mut monitor = self.root_monitor.lock()?;
-            if monitor.free_list_dirty {
-                self.disc().sync()?;
-            }
-            let mut pag = self.disc.load_page(self.page)?.clone_write();
-            lock.write_list(&mut pag)?;
-            self.disc.flush_page(&pag)?;
-            monitor.free_list_dirty = true;
+            let page = self.disc.load_page(self.page)?.clone_write();
+            let mut buffer = lock.write_list().to_vec();
+            let holder = &mut monitor.free_list_holder;
+            self.write_root(page, holder, &mut buffer, ALLOCATOR_ROOT_PAGE_VERSION)?;
             // I do not do the disk sync here because is every time done by the caller.
             lock.to_flush = false;
         }
         Ok(())
     }
 
-    pub fn write_address_root(&self, root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PERes<u8> {
+    pub fn write_address_root(&self, root: Page, buffer: &mut [u8], version: u8) -> PERes<()> {
         let mut monitor = self.root_monitor.lock()?;
-        if monitor.address_dirty {
-            self.disc().sync()?;
-        }
-        let order = self.write_root_page(root, buffer, version, last_flush)?;
-        monitor.address_dirty = true;
-        Ok(order)
+        self.write_root(root, &mut monitor.address_holder, buffer, version)
     }
-    pub fn write_juornal_root(&self, root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PERes<u8> {
+    pub fn write_journal_root(&self, root: Page, buffer: &mut [u8], version: u8) -> PERes<()> {
         let mut monitor = self.root_monitor.lock()?;
-        if monitor.journal_dirty {
-            self.disc().sync()?;
-        }
-        let order = self.write_root_page(root, buffer, version, last_flush)?;
-        monitor.journal_dirty = true;
-        Ok(order)
+        self.write_root(root, &mut monitor.journal_holder, buffer, version)
     }
 
-    fn write_root_page(&self, mut root: Page, buffer: &mut [u8], version: u8, last_flush: u8) -> PERes<u8> {
-        let order = write_root_page(&mut root, buffer, version, last_flush)?;
+    fn write_root(&self, root: Page, holder: &mut RootPageHolder, buffer: &mut [u8], version: u8) -> PERes<()> {
+        holder.page = root.get_index();
+        holder.version = version;
+        holder.buffer = Some(Vec::from(buffer));
+        holder.dirty = true;
+        Ok(())
+    }
+
+    fn write_root_page(root: &mut Page, holder: &mut Counter, mut buffer: Vec<u8>, version: u8) -> PERes<()> {
+        let last_flush = holder.flush_counter;
+        let order = write_root_page(root, &mut buffer, version, last_flush)?;
+        holder.flush_counter = order;
+        Ok(())
+    }
+
+    fn write_root_page_info(&self, mut info: RootWriteInfo, holder: &mut Counter) -> PERes<()> {
+        let mut root = self.write_page(info.page)?;
+        let last_flush = holder.flush_counter;
+        let order = write_root_page(&mut root, &mut info.buffer, info.version, last_flush)?;
         self.flush_page(root)?;
-        Ok(order)
+        holder.flush_counter = order;
+        Ok(())
+    }
+
+    pub fn read_root_journal(&self, page: &mut ReadPage, buffer_size: usize) -> Vec<u8> {
+        let mut monitor = self.root_monitor.lock().unwrap();
+        let mut counter_monitor = self.flush_count.lock().unwrap();
+        Allocator::read_root_page_int(
+            page,
+            buffer_size,
+            &mut monitor.journal_holder,
+            &mut counter_monitor.journal,
+        )
+    }
+
+    pub fn read_root_address(&self, page: &mut ReadPage, buffer_size: usize) -> Vec<u8> {
+        let mut monitor = self.root_monitor.lock().unwrap();
+        let mut counter_monitor = self.flush_count.lock().unwrap();
+        Allocator::read_root_page_int(
+            page,
+            buffer_size,
+            &mut monitor.address_holder,
+            &mut counter_monitor.address,
+        )
+    }
+
+    fn read_root_page_int(
+        page: &mut ReadPage,
+        buffer_size: usize,
+        holder: &mut RootPageHolder,
+        counter: &mut Counter,
+    ) -> Vec<u8> {
+        let mut buffer_0 = vec![0; buffer_size];
+        let mut buffer_1 = vec![0; buffer_size];
+        InfallibleRead::read_exact(page, &mut buffer_0);
+        InfallibleRead::read_exact(page, &mut buffer_1);
+        let (flush, first) = double_buffer_check(&buffer_0, &buffer_1);
+        let buffer = if first { buffer_0 } else { buffer_1 };
+        holder.buffer = Some(buffer.clone());
+        counter.flush_counter = flush;
+        buffer
     }
 
     pub fn flush_root_page(&self, page: Page) -> PERes<()> {
@@ -421,11 +497,28 @@ impl Allocator {
     }
 
     pub fn disc_sync(&self) -> PERes<()> {
-        let mut monitor = self.root_monitor.lock()?;
-        self.disc.sync()?;
-        monitor.free_list_dirty = false;
-        monitor.journal_dirty = false;
-        monitor.address_dirty = false;
+        let free_list_data;
+        let journal_data;
+        let address_data;
+        {
+            let mut monitor = self.root_monitor.lock()?;
+            free_list_data = monitor.free_list_holder.write_data();
+            journal_data = monitor.journal_holder.write_data();
+            address_data = monitor.address_holder.write_data();
+        }
+        {
+            let mut fm = self.flush_count.lock()?;
+            if let Some(info) = free_list_data {
+                self.write_root_page_info(info, &mut fm.free_list)?;
+            }
+            if let Some(info) = journal_data {
+                self.write_root_page_info(info, &mut fm.journal)?;
+            }
+            if let Some(info) = address_data {
+                self.write_root_page_info(info, &mut fm.address)?;
+            }
+            self.disc.sync()?;
+        }
         Ok(())
     }
 
@@ -458,7 +551,7 @@ mod tests {
     use super::{Allocator, Cache, FreeList};
     use crate::{
         config::Config,
-        discref::{DiscRef, PageOps, ReadPage},
+        device::{FileDevice, PageOps, ReadPage},
     };
     use std::{sync::Arc, time::Duration};
     use tempfile::Builder;
@@ -472,7 +565,7 @@ mod tests {
             .unwrap()
             .reopen()
             .unwrap();
-        let disc = Box::new(DiscRef::new(file).unwrap());
+        let disc = Box::new(FileDevice::new(file).unwrap());
         let (_, allocator) = Allocator::init(disc, &Config::new()).unwrap();
         // This is needed to avoid the 0 page
         allocator.allocate(10).unwrap();
@@ -507,7 +600,7 @@ mod tests {
             .unwrap()
             .reopen()
             .unwrap();
-        let disc = Box::new(DiscRef::new(file).unwrap());
+        let disc = Box::new(FileDevice::new(file).unwrap());
         let (_, allocator) = Allocator::init(disc, &Config::new()).unwrap();
         // This is needed to avoid the 0 page
         allocator.allocate(2).unwrap();
