@@ -490,10 +490,12 @@ where
     fn get(&mut self, k: &K) -> PERes<Option<Value<V>>> {
         Ok(if let Some(node) = self.get_root()? {
             let mut cur_node = node;
+            let mut reuse = None;
             loop {
-                match self.load(&cur_node)? {
+                match self.load_with(&cur_node, reuse)? {
                     Node::Node(n) => {
                         cur_node = n.find(k).node_ref;
+                        reuse = Some(n);
                     }
                     Node::Leaf(leaf) => {
                         break leaf.find(k).map(|el| el.1).ok();
@@ -509,8 +511,9 @@ where
         let mut path = Vec::new();
         let mut iter = if let Some(mut cur_node) = self.get_root()? {
             path.push((0, cur_node));
+            let mut reuse = None;
             loop {
-                match self.load(&cur_node)? {
+                match self.load_with(&cur_node, reuse)? {
                     Node::Node(n) => {
                         let value = match first {
                             Bound::Included(f) => {
@@ -525,6 +528,7 @@ where
                         };
                         cur_node = value.1;
                         path.push(value);
+                        reuse = Some(n);
                     }
                     Node::Leaf(leaf) => {
                         break PageIter {
@@ -580,8 +584,9 @@ where
         let mut path = Vec::new();
         let mut iter = if let Some(mut cur_node) = self.get_root()? {
             path.push((0, cur_node));
+            let mut reuse = None;
             loop {
-                match self.load(&cur_node)? {
+                match self.load_with(&cur_node, reuse)? {
                     Node::Node(n) => {
                         let value = match last {
                             Bound::Included(f) => {
@@ -599,6 +604,7 @@ where
                         };
                         cur_node = value.1;
                         path.push(value);
+                        reuse = Some(n);
                     }
                     Node::Leaf(leaf) => {
                         break PageIterBack {
