@@ -1,4 +1,4 @@
-use persy::{Config, PRes, Persy};
+use persy::{Config, GenericError, Persy};
 use std::sync::{Condvar, Mutex};
 use tempfile::Builder;
 
@@ -17,7 +17,7 @@ impl CountDown {
         }
     }
 
-    pub fn wait(&self) -> PRes<bool> {
+    pub fn wait(&self) -> Result<bool, GenericError> {
         let guard = self.lock.lock()?;
         if *guard != 0 {
             let _ = self.cond.wait(guard)?;
@@ -25,7 +25,7 @@ impl CountDown {
         Ok(true)
     }
 
-    pub fn count_down(&self) -> PRes<()> {
+    pub fn count_down(&self) -> Result<(), GenericError> {
         let mut count = self.lock.lock()?;
         *count = (*count) - 1;
         if *count == 0 {
